<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/12 14:37
 */
namespace api\controllers;

use common\classes\MysqlAccess;
use common\models\DeviceConfig;
use common\models\DeviceList;
use common\models\UserDevices;
use common\models\Users;
use common\models\VDevice;
use Yii;
use yii\db\Exception;
use yii\db\Query;
use yii\filters\VerbFilter;
use common\classes\Error;

/**
 * Site controller
 */
class DeviceController extends BaseController
{
    public static function userDevices($uid, $pri) {
        return UserDevices::find()
            ->select(['DevID', 'Pri'])
            ->asArray()
            ->where(['UserID' => $uid])
            ->indexBy('DevID')
            ->andWhere("Pri & $pri = $pri")
            ->all();
    }
    /**
     * get user's mgr devices(db Query)
     * @return $this|null|\yii\db\ActiveQuery
     */
    private function newQuery(){
        $user = Yii::$app->user->identity;
        if($user->Mgr_Privilege == 1 || $user->User_ID == 0) {
            $query = VDevice::find();
        }
        else {
            $devices = self::userDevices($user->User_ID, 2);
            $ids = [];
            foreach ($devices as $dev)
                $ids[] = $dev['DevID'];
            $query = VDevice::find()->where(['in', 'id', $ids]);
        }
        return $query->orderBy(['ipaddress' => SORT_DESC, 'name' => SORT_ASC, 'id' => SORT_ASC]);
    }

    private function mgrPriCheck($id){
        $user = Yii::$app->user->identity;
        if($user->Mgr_Privilege == 1 || $user->User_ID == 0) {
            return true;
        }
        else {
            $count = UserDevices::find()
                ->where(['UserID' => $user->User_ID, 'DevID' => $id])
                ->andWhere('Pri & 2 = 2')
                ->count();
            return ($count == 1);
        }
    }
    /*
     * get All Device Class
     */
    public function actionGetConfig(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $config = DeviceConfig::readConfig($data['id']);
        if(is_null($config))
            return $this->response(Error::STATUS_FAILED);
        else
            return $this->response(Error::STATUS_SUCCESS, $config);
    }

    public function actionSetConfig(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || empty($data['id']) || !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $config = new DeviceConfig();
        if($config->load(Yii::$app->getRequest()->post(), 'data') && $config->save()) {
            $devices = Yii::$app->db->createCommand('select Dev_Name from DeviceList where DevID in ( ' . $data['id'] .')')
                ->queryAll();
            $this->logDetail['ele/id'] = '';
            foreach ($devices as $item) {
                $this->logDetail['ele/id'] .=  $item['Dev_Name'] . "\n";
            }
            return $this->response(Error::STATUS_SUCCESS);
        }
        else
            return $this->response(Error::STATUS_FAILED);
    }

    public function actionList(){
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);
        try{
            $query = $this->newQuery();
            if($query == null)
                return $this->response(Error::STATUS_SUCCESS, []);
            $query->select(['id', 'name', 'parent'])->orderBy(['Parent' => SORT_ASC]);
            if (!is_null($data)) {
                if (isset($data['class']) && is_numeric($data['class'])) {
                    if (isset($data['recursion']) && $data['recursion'] === true) {
                        $query->andWhere('sp_dev_belong_group(id, ' . $data['class'] . ')');
                    } else {
                        $query->andWhere(['parent' => $data['class']]);
                    }
                } else if (!isset($data['recursion']) || $data['recursion'] !== true) {
                    $query->andWhere(['parent' => null]);
                }
            }
            $devices = $query->all();
            return $this->response(Error::STATUS_SUCCESS, $devices);
        }catch(Exception $e){
            return $this->response(Error::STATUS_UNKNOW);
        }
    }

    public function actionStatus(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $query = $this->newQuery();
        if($query == null)
            return $this->response(Error::STATUS_SUCCESS, []);
        $this->listQuery($query, $data);
        if(isset($data['devices']) && preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices']))
            $query->andWhere(['in', 'id', explode(',', $data['devices'])]);
        else if(isset($data['class']) && is_numeric($data['class'])) {
            if(isset($data['recursion']) && $data['recursion'] === true){
                $query->andWhere('sp_dev_belong_group(id, ' . $data['class'].')');
            }
            else {
                $query->andWhere(['parent' => $data['class']]);
            }
        }
        else if(!isset($data['recursion']) || $data['recursion'] !== true){
            $query->andWhere(['parent' => NULL]);
        }
        $devices = $query->all();
        return $this->response(Error::STATUS_SUCCESS, $devices);
    }

    public function actionControl(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['devices']) || !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['devices']))
            return $this->response(Error::STATUS_PARAM_ERROR, $data);

        $this->logDetail['ele/id'] = '';
        $device = new VDevice();
        if($device->load(Yii::$app->request->post(), 'data')) {
            $device->save();
            $devices = Yii::$app->db->createCommand('select Dev_Name from DeviceList where DevID in ( ' . $data['devices'] .')')
                ->queryAll();
            foreach ($devices as $item) {
                $this->logDetail['ele/id'] .= $item['Dev_Name'] ."\n";
            }
            if (isset($device->power))
                $this->logDetail['ele/power'] = $device->power;
            if (isset($this->pc))
                $this->logDetail['ele/pc'] = $device->pc;
            if (isset($this->projector))
                $this->logDetail['ele/projector'] = $device->projector;
            if (isset($this->audioMute))
                $this->logDetail['ele/audioMute'] = $device->audioMute;
            if (isset($this->micMute))
                $this->logDetail['ele/micMute'] = $device->micMute;
            if (isset($this->micVol) && is_numeric($this->micVol))
                $this->logDetail['ele/micVol'] = $device->micVol;
            if (isset($this->audioVol) && is_numeric($this->audioVol))
                $this->logDetail['ele/audioVol'] = $device->audioVol;
            if (isset($this->projectorSrc))
                $this->logDetail['ele/projectorSrc'] = $device->projectorSrc;
            if (isset($this->iotStatus) && strlen($this->iotStatus) == 32 && ctype_xdigit($this->iotStatus))
                $this->logDetail['ele/iotStatus'] = $device->iotStatus;
            return $this->response(Error::STATUS_SUCCESS);
        }
        else
            return $this->response(Error::STATUS_FAILED);
    }

    public function actionBrdMute($id, $mute) {
        $query = $this->newQuery();
        if($query == null)
            return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
        $ids = preg_split('/,/', $id);
        if(empty($ids))
            return $this->response(Error::STATUS_SUCCESS);
        foreach ($ids as $id) {
            $access = new MysqlAccess($id);
            if(is_null($access))
                continue;
            if ($mute == 'on')
                $access->deviceCtrl(MysqlAccess::CMD_BRD_MUTE, 0);
            else
                $access->deviceCtrl(MysqlAccess::CMD_BRD_MUTE, 1);
        }
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionTasks(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        if(!$this->mgrPriCheck($data['id']))
            return $this->response(Error::STATUS_FORBID);
        if(isset($data['date'])){
            $start = $data['date'] . ' 00:00:00';
            $end = $data['date'] . ' 23:59:59';
        }
        else{
            $start = date('Y-m-d 00:00:00');
            $end = date('Y-m-d 23:59:59');
        }
        try{
            $ret = Yii::$app->getDb()
                ->createCommand("call sp_get_dev_scheduled_tasks('" . $data['id'] . "', '$start', '$end');")
                ->queryAll();
            return $this->response(Error::STATUS_SUCCESS, $ret);
        }catch (\Exception $e){
            return $this->response(Error::STATUS_FAILED, $e);}
    }

    public function actionCameraList($id){
        $model = DeviceList::find()->select(['DevID', 'RtspUrl', 'RtspUrl1'])
            ->where(['DevID' => $id])
            ->one();
        return $this->response(Error::STATUS_SUCCESS, $model);
    }

    private function deviceDelete($id){
        Yii::$app->getDb()
            ->createCommand("call sp_delete_device('" . $id . "');")
            ->execute();
        return VDevice::find()->where(['id' => $id])->count();
    }
    /*
     * delete offline device
     */
    public function actionDelete(){
        $data = Yii::$app->getRequest()->post('data');
        if(is_null($data) || !isset($data['id']) ||
            !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['id']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $success = [];
        try{
            $ids = explode(',', $data['id']);
            foreach($ids as $id){
                if(!$this->mgrPriCheck($id))
                    continue;
                $device = VDevice::findOne(['id' => $id]);
                if(is_null($device))
                    continue;
                if(is_null($device->ipaddress) && $this->deviceDelete($id) == 0)
                    $success[] = $id;
            }
        }catch (\Exception $e){
            return $this->response(Error::STATUS_FAILED);
        }

        $devices = DeviceList::find()->select(['Dev_Name'])
            ->where(['in', 'DevID', $success])->all();
        $this->logDetail['ele/id'] = '';
        foreach ($devices as $item){
            $this->logDetail['ele/id'] .= $item->Dev_Name . "\n";
        }
        if(empty($this->logDetail['ele/id']))
            unset($this->logDetail['ele/id']);
        return $this->response(Error::STATUS_SUCCESS, ['devices' => $success]);
    }

    /**
     * device (config/name+class/task+mode/privilege) copy
     */
    public function actionCopy(){
        $data = Yii::$app->getRequest()->post('data');
        if (is_null($data)
            || !isset($data['src']) || !preg_match('/^[0-9]+[0-9]$/', $data['src'])
            || !isset($data['dst'])  || !preg_match('/^[0-9][0-9,]+[0-9]$/', $data['dst'])
            || !isset($data['mask']) || !is_numeric($data['mask']))
            return $this->response(Error::STATUS_PARAM_ERROR);

        $this->logDetail['ele/src'] = $data['src'];
        $this->logDetail['ele/dst'] = $data['dst'];
        if(!$this->mgrPriCheck($data['src']))
            return $this->response(Error::STATUS_FORBID);
        $srcDevice = VDevice::findOne(['id' => $data['src']]);
        if(is_null($srcDevice))
            return $this->response(Error::STATUS_DEVICE_NOT_EXISTS);
        if (($data['mask'] & 0x01) == 0)
            $data['mask'] &= 0xf5;
        $device = explode(',', $data['dst']);
        if($device === false || count($device) == 0)
            return $this->response(Error::STATUS_SUCCESS);
        $sql = "select func_dulicate_device('" . $data['src'] . "', :dst, :mask) as ret;";
        $cmd = Yii::$app->getDb()->createCommand($sql);
        $cmd = $cmd->bindParam(':dst', $dst1);
        $cmd = $cmd->bindParam(':mask', $mask);
        $success = [];
        foreach ($device as $dst) {
            if($dst == $data['src'])
                continue;
            if(!$this->mgrPriCheck($dst))
                continue;
            $dst1 = $dst;
            //Clear bit2, to disable task data copy
            if(substr($data['src'], 0, 1) != substr($dst, 0, 1))
                $mask = $data['mask'] & 0xfb;
            else
                $mask = $data['mask'];
            try {
                $out = $cmd->queryOne();
            } catch (\Exception $e) {
                continue;
            }
            if($out['ret'] !== "0")
                $success[] = $dst;
        }
        //offline and select all bit(0x0f), delete src device
        if (is_null($srcDevice->ipaddress) && ($data['mask'] & 0x1f) == 0x1f && count($success) >= 1)
            $this->deviceDelete($data['src']);
        return $this->response(Error::STATUS_SUCCESS, $success);
    }
}
