<?php
namespace api\controllers;

use common\classes\Common;
use common\classes\HttpRequest;
use common\models\BaseInfo;
use common\models\System;
use common\models\Users;
use Yii;
use yii\filters\VerbFilter;
use yii\filters\AccessControl;
use common\models\LoginForm;
use common\classes\Error;
use common\models\Cache;
use common\models\UserDevices;
use yii\helpers\ArrayHelper;
use common\models\Users as User;
use yii\web\ForbiddenHttpException;

/**
 * User controller
 */
class UserController extends BaseController
{
    public $postRequest = ['list', 'update', 'new', 'delete', 'auth', 'update-mgr', 'unionid-bind', 'unionid-unbind'];

    /**
     * get detail infomation for current login user
     */
    public function actionViewMgr(){
        $user = Yii::$app->getUser();
        $user = User::findIdentity($user->getId());
        if(is_null($user))
            return $this->response(Error::STATUS_FAILED);
        // $user = User::findIdentity(0);
        $ret = $user->toArray(['User_Name', 'User_Desc', 'IcCard', 'Phone', 'User_Sn', 'UnionId']);
        if (!is_null($ret['UnionId']))
            $ret['UnionId'] = 'bind';
        else
            $ret['UnionId'] = 'unbind';
        return $this->response(Error::STATUS_SUCCESS, $ret);
    }

    public function actionUpdateMgr(){
        if (!array_key_exists('User_Desc', $this->data) || !array_key_exists('IcCard', $this->data)
            || !array_key_exists('Phone', $this->data) || !array_key_exists('User_Sn', $this->data)
            || !isset($this->data['Pass_Word']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $user = Yii::$app->getUser();
        $user = User::findIdentity($user->getId());
        if (is_null($user))
            return $this->response(Error::STATUS_FAILED);
        if ($user->Pass_Word != User::mysql_password($this->data['Pass_Word']))
            return $this->response(Error::STATUS_USER_PASSWORD_ERROR);
        if (isset($this->data['New_Pass_Word']))
            $user->Pass_Word = User::mysql_password($this->data['New_Pass_Word']);
        $user->User_Desc = isset($this->data['User_Desc']) ? $this->data['User_Desc'] : null;
        $user->IcCard = (!empty($this->data['IcCard']))? trim($this->data['IcCard']) : null;
        $user->Phone = (!empty($this->data['Phone']))? trim($this->data['Phone']) : null;
        $user->User_Sn = (!empty($this->data['User_Sn']))? trim($this->data['User_Sn']) : null;
        if(!empty($user->IcCard)) {
            $count = User::find()->where(['not', ['User_Name' => $user->User_Name]])
                ->andWhere(['IcCard' => $user->IcCard])->count();
            if($count > 0)
                return $this->response(Error::STATUS_USER_ICCARD_EXIST);
        }
        if (!empty($user->User_Sn)) {
            $count = User::find()->where(['not', ['User_Name' => $user->User_Name]])
                ->andWhere(['User_Sn' => $user->User_Sn])->count();
            if ($count > 0)
                return $this->response(Error::STATUS_USER_USERSN_EXIST);
        }
        if($user->save())
            return $this->response(Error::STATUS_SUCCESS);
        else
            return $this->response(Error::STATUS_FAILED);
    }

    /**
     * bind new unionid
     */
    public function actionUnionidBind()
    {
        if (!Yii::$app->getUser()->identity->cloudLogin)
            return $this->response(Error::STATUS_FAILED);
        if (!isset($this->data['unionid'])
            || !isset($this->data['token']) || strlen($this->data['token']) < 32)
            return $this->response(Error::STATUS_PARAM_ERROR);
        $hashUnionid = hash('sha256', Common::UNIONID_MD5_PRESTR . $this->data['unionid']);
        $cred = Yii::$app->getCache()->get(Common::LOGIN_TOKEN_CACHE_SALT . $this->data['token']);
        if ($cred === false || !isset($cred['user']))
            throw new ForbiddenHttpException();
        $user = User::findOne(['UnionId' => $hashUnionid]);
        if(!is_null($user))
            return $this->response(Error::STATUS_USER_UNIONID_BOUND_OTHER);
        $user = User::findIdentity($cred['user']->id);
        if (is_null($user))
            return $this->response(Error::STATUS_FAILED);
        if (!empty($user->UnionId))
            return $this->response(Error::STATUS_USER_UNIONID_BOUND);
        $user->UnionId = $hashUnionid;
        if ($user->save()) {
            $ret = [
                'pid' => $user->User_ID,
                'username' => $user->User_Name,
                'fullname' => $user->User_Desc,
                'phone' => $user->Phone,
                'unionid' => $this->data['unionid']
            ];
            Yii::$app->udp->bind($user, '绑定成功');
            return $this->response(Error::STATUS_SUCCESS, $ret);
        }
        return $this->response(Error::STATUS_FAILED);
    }

    /**
     * unbind unionid
     */
    public function actionUnionidUnbind()
    {
        if (!Yii::$app->getUser()->identity->cloudLogin)
            return $this->response(Error::STATUS_FAILED);
        if (!isset($this->data['token']) || strlen($this->data['token']) < 32)
            return $this->response(Error::STATUS_PARAM_ERROR);
        $cred = Yii::$app->getCache()->get(Common::LOGIN_TOKEN_CACHE_SALT . $this->data['token']);
        if ($cred === false || !isset($cred['user']))
            return $this->response(Error::STATUS_ERROR_401);
        $user = User::findIdentity($cred['user']->id);
        if (is_null($user))
            return $this->response(Error::STATUS_FAILED);
        $user->UnionId = null;
        if ($user->save()) {
            $ret = [
                'pid' => $user->User_ID
            ];
            Yii::$app->udp->bind($user, '解除绑定');
            return $this->response(Error::STATUS_SUCCESS, $ret);
        } else
            return $this->response(Error::STATUS_FAILED);
    }


    public function actionDpmList() {
        $dpms = User::find()->select(['Dpm_Name'])
            ->where(['not', ['Dpm_Name' => null]])
            ->andWhere('LENGTH(trim(Dpm_Name)) > 0')
            ->groupBy(['Dpm_Name'])
            ->all();
        return $this->response(Error::STATUS_SUCCESS, $dpms);
    }

    public function actionList() {
        if(!isset($this->data['limit']) || $this->data['limit'] > 100)
            $this->data['limit'] = 100;
        if (!isset($this->data['page']))
            $this->data['page'] = 0;

        $query = User::find()->select([
                        'User_ID', 'User_Name', 'User_Desc', 'User_Sn', 'IcCard', 'IC_Status', 'User_Type', 'Dpm_Name','Create_Type',
                        "(case (isnull(UnionId)) when true then 'unbind' else 'bind' end) as UnionId"
                    ])
                    ->where('User_ID > 0');
        $count = $this->listQuery($query, $this->data);
        $users = $query->all();
        return $this->response(Error::STATUS_SUCCESS, [
            'count' => $count,
            'page' => $this->data['page'],
            'limit' => $this->data['limit'],
            'users' => $users
        ]);
    }

    public function actionAuth()
    {
        if(!$this->isAdmin)
            unset($this->data['mgrPrivilege'], $this->data['mgrDevices']);
        if(!isset($this->data['users']) ||
            (isset($this->data['devPrivilege']) && $this->data['devPrivilege'] == 0 && !isset($this->data['useDevices'])) ||
            (isset($this->data['mgrPrivilege']) && $this->data['mgrPrivilege'] == 0 && !isset($this->data['mgrDevices'])))
            return $this->response(Error::STATUS_PARAM_ERROR, $this->data);
        if(!preg_match('/^[0-9][0-9,]*$/', $this->data['users']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        $db = Yii::$app->getDb();
        $users = $db->createCommand('select User_Desc from Users where User_ID in (' . $this->data['users'] . ')')
            ->queryAll();
        $userStr = '';
        foreach ($users as $user) {
            $userStr .= $user['User_Desc'] . "\n";
        }
        if(!empty($userStr))
            $this->logDetail['users'] = mb_substr($userStr, 0, 980);

        if(isset($this->data['devPrivilege'])) {
            if ($this->data['devPrivilege'] == 1) {
                $db->createCommand('update Users set Dev_Privilege = 1 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
            } else {
                $db->createCommand('update Users set Dev_Privilege = 0 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
                if (!empty($this->data['useDevices']) && is_array($this->data ['useDevices'])) {
                    $devices = implode(',', $this->data['useDevices']);
                    if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                        return $this->response(Error::STATUS_PARAM_ERROR);
                    $db->createCommand("update UserDevices set Pri = Pri & 0xfe where UserID in (" . $this->data['users'] . ") and DevID not in ($devices);")
                        ->query();
                    $db->createCommand("insert into UserDevices (UserID, DevID, Pri)
                    select User_ID, DevID ,1  from DeviceList, Users where User_ID in (" . $this->data['users'] . ") and DevID in ($devices)
                    on duplicate key update Pri = Pri | 1;")
                        ->query();
                } else
                    $db->createCommand('update UserDevices set Pri = Pri & 0xfe where UserID in ( ' . $this->data['users'] . ');')
                        ->query();
            }
        }

        if(isset($this->data['mgrPrivilege'])) {
            if ($this->data['mgrPrivilege'] == 1) {
                $db->createCommand('update Users set Mgr_Privilege = 1 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
            } else {
                $db->createCommand('update Users set Mgr_Privilege = 0 where User_ID in ( ' . $this->data['users'] . ');')
                    ->query();
                if (!empty($this->data['mgrDevices']) && is_array($this->data['mgrDevices'])) {
                    $devices = implode(',', $this->data['mgrDevices']);
                    if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                        return $this->response(Error::STATUS_PARAM_ERROR);
                    $db->createCommand("update UserDevices set Pri = Pri & 0xfd where UserID in (" . $this->data['users'] . ") and DevID not in ($devices);")
                        ->query();
                    $db->createCommand("insert into UserDevices (UserID, DevID, Pri)
                    select User_ID, DevID ,2  from DeviceList, Users where User_ID in (" . $this->data['users'] . ") and DevID in ($devices)
                    on duplicate key update Pri = Pri | 2;")
                        ->query();

                } else
                    $db->createCommand('update UserDevices set Pri = Pri & 0xfd where UserID in (' . $this->data['users'] . ');')
                        ->query();
            }
        }

        $db->createCommand('delete from UserDevices where Pri = 0;')->query();
        return $this->response(Error::STATUS_SUCCESS);
    }

    public function actionView($id)
    {
        if (Yii::$app->getUser()->getId() !== 0 && $id === 0)
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        $user = User::findOne(['User_ID' => $id]);
        if(is_null($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        if (!is_null($user->UnionId))
            $user->UnionId = 'bind';
        else
            $user->UnionId = 'unbind';
        $useDevices = [];
        $mgrDevices = [];
        if($user->Dev_Privilege != 1) {
            $items = UserDevices::find()
                ->select(['DevID as device'])
                ->where(['UserID' => $id])
                ->andWhere('Pri & 1 = 1')
                ->asArray()
                ->all();
            foreach($items as $item)
                $useDevices[] = $item['device'];
        }
        if ($user->Mgr_Privilege != 1) {
            $items = UserDevices::find()
                ->select(['DevID as device'])
                ->where(['UserID' => $id])
                ->andWhere('Pri & 2 = 2')
                ->asArray()
                ->all();
            foreach ($items as $item)
                $mgrDevices[] = $item['device'];
        }
        return $this->response(Error::STATUS_SUCCESS, [
            'user' => $user,
            "useDevices" => $useDevices,
            "mgrDevices" => $mgrDevices,
        ]);
    }

    public function actionDelete()
    {
        if (!isset($this->data['User_ID']) || !preg_match('/^[0-9][0-9,]*$/', $this->data['User_ID']))
            return $this->response(Error::STATUS_PARAM_ERROR);
        //'admin' can't to be deleted
        $db = Yii::$app->getDb();
        $users = $db->createCommand('select User_Desc from Users where User_ID in (' . $this->data['User_ID'] . ')')
            ->queryAll();
        $userStr = '';
        foreach ($users as $user) {
            $userStr .= $user['User_Desc'] . "\n";
        }
        if(!empty($userStr))
            $this->logDetail['users'] = mb_substr($userStr, 0, 980);
        if ($this->isAdmin)
            $db->createCommand('delete from Users where User_ID in (' . $this->data['User_ID'] . ') and User_ID > 0;')
                ->execute();
        else
            $db->createCommand('delete from Users where User_ID in (' . $this->data['User_ID'] . ') and User_ID > 0 and User_Name is null;')
                ->execute();

        $ret = $db->createCommand('select User_ID from Users where User_ID in (' . $this->data['User_ID'] . ')')
            ->queryAll();
        return $this->response(Error::STATUS_SUCCESS, ['fail' => $ret]);
    }

    private function UserNP($user)
    {
        unset($this->data['user']['UnionId']);
        unset($this->data['user']['User_ID']);
        unset($this->data['user']['Create_Type']);

        if($user->isNewRecord)
            $id = -1;
        else
            $id = $user->User_ID;
        if(!$this->isAdmin ) {
            unset ($this->data['user']['User_Name']);
            unset ($this->data['user']['New_Pass_Word']);
            unset($this->data['user']['privilege']);
            unset($this->data['user']['UsbKey']);
            unset($this->data['user']['Mgr_Privilege']);
        }
        else if(!empty($this->data['user']['User_Name']) ) {
            $count = User::find()->where(['not', ['User_ID' => $id]])
                ->andWhere(['User_Name' => $this->data['user']['User_Name']])
                ->count();
            if($count > 0)
                return $this->response(Error::STATUS_USER_USER_NAME_EXIST);
        } else if(isset($this->data['user']['User_Name']))
            $this->data['user']['User_Name'] = null;

        if(!empty($this->data['user']['IcCard']))
        {
            $count = User::find()->where(['not', ['User_ID' => $id]])
                ->andWhere(['IcCard' => $this->data['user']['IcCard']])
                ->count();
            if($count > 0)
            return $this->response(Error::STATUS_USER_ICCARD_EXIST);
        } else if(isset($this->data['user']['IcCard']))
            $this->data['user']['IcCard'] = null;

        if (!empty($this->data['user']['User_Sn'])) {
            $count = User::find()->where(['not', ['User_ID' => $id]])
                ->andWhere(['User_Sn' => $this->data['user']['User_Sn']])
                ->count();
            if ($count > 0)
                return $this->response(Error::STATUS_USER_USERSN_EXIST);
        } else if(isset($this->data['user']['User_Sn']))
            $this->data['user']['User_Sn']  = null;

        if($this->isAdmin) {
            if (isset($this->data['user']['privilege']) && is_numeric($this->data['user']['privilege']) && ($this->data['user']['privilege'] & 0x100) == 0x100) {
                if (!empty($this->data['user']['UsbKey'])) {
                    $count = User::find()->where(['not', ['User_ID' => $id]])
                        ->andWhere(['UsbKey' => $this->data['user']['UsbKey']])
                        ->count();
                    if ($count > 0)
                        return $this->response(Error::STATUS_USER_USBKEY_EXIST);
                } else if(isset($this->data['user']['UsbKey']))
                    $this->data['user']['UsbKey'] = null;
            }
        }
        $oldPri = $user->privilege;

        if ($user->load($this->data, 'user')) {
            if (!empty($this->data['user']['New_Pass_Word']))
                $user->Pass_Word = User::mysql_password($this->data['user']['New_Pass_Word']);
            if ($user->save()) {
                $this->logTitle['title1'] = $user->User_Desc;
                $db = Yii::$app->getDb();
                if($user->isNewRecord)
                    $user = User::findOne(['User_ID' => $db->getLastInsertID()]);
                do {
                    Yii::$app->cache->delete(Common::USER_INFO_SALT . $user->User_ID);
                    if ($user->Dev_Privilege != 1 && isset($this->data['useDevices'])) {
                        if (!empty($this->data['useDevices']) && is_array($this->data['useDevices'])) {
                            $devices = implode(',', $this->data['useDevices']);
                            if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                                break;
                            $db->createCommand(
                                "update UserDevices set Pri = Pri & 0xfe where UserID=:userid and DevID not in ($devices);",
                                [':userid' => $user->User_ID]
                            )
                                ->query();
                            $db->createCommand('insert into UserDevices (UserID, DevID, Pri)
                                select :userid, DevID ,1 from DeviceList where DevID in (' . $devices . ')
                                on duplicate key update Pri = Pri | 1;', [':userid' => $user->User_ID])
                                ->query();
                        } else
                            $db->createCommand('update UserDevices set Pri = Pri & 0xfe where UserID=:userid;', [':userid' => $user->User_ID])
                                ->query();
                    }
                    if($this->isAdmin && $user->Mgr_Privilege != 1 && isset($this->data['mgrDevices'])) {
                        if(!empty($this->data['mgrDevices']) && is_array($this->data['mgrDevices'])){
                            $devices = implode(',', $this->data['mgrDevices']);
                            if (!preg_match('/^[0-9][0-9,]+[0-9]$/', $devices))
                                break;
                            $db->createCommand(
                                'update UserDevices set Pri = Pri & 0xfd where UserID=:userid and DevID not in (' . $devices . ');',
                                [':userid' => $user->User_ID]
                            )
                                ->query();
                            $db->createCommand('insert into UserDevices (UserID, DevID, Pri)
                            select :userid, DevID ,2 from DeviceList where DevID in (' . $devices . ')
                            on duplicate key update Pri = Pri | 2;', [':userid' => $user->User_ID])
                                ->query();
                        }
                        else
                            $db->createCommand('update UserDevices set Pri = Pri & 0xfd where UserID=:userid;', [':userid' => $user->User_ID])
                                ->query();
                    }

                    $db->createCommand('delete from UserDevices where Pri = 0;')->query();
                } while (false);
                return $this->actionView($user->User_ID);
            }
        }
        return $this->response(Error::STATUS_FAILED, ['error' => $user->errors]);
    }

    public function actionUpdate($id)
    {
        if (!isset($this->data['user']))
            return $this->response(Error::STATUS_FAILED);
        if (Yii::$app->getUser()->getId() !== 0 && $id === 0)
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        $user = User::findOne(['User_ID' => $id]);
        if (is_null($user))
            return $this->response(Error::STATUS_USER_NOT_EXISTS);
        return $this->userNP($user);
    }

    public function actionNew()
    {
        if (!isset($this->data['user']))
            return $this->response(Error::STATUS_FAILED);
        $user = new User();
        return $this->userNP($user);
    }
}
