<?php

namespace common\models;

use Yii;

/**
 * This is the model class for table "DeviceList".
 *
 * @property string $DevID
 * @property string $Dev_Name
 * @property integer $Parent
 * @property integer $DisableParentSchedule
 * @property resource $Dev_Description
 * @property resource $Dev_Configuration
 * @property string $CameraIpAddr
 * @property integer $CameraPort
 * @property string $CameraUser
 * @property string $CameraPasswd
 * @property integer $CameraType
 * @property integer $CameraChannel
 * @property string $RtspUrl
 * @property integer $Qrcode
 * @property string $RtspUrl1
 *
 * @property BrdModeMember[] $brdModeMembers
 * @property BroadcastMode[] $brdModes
 * @property DevCtrlSchedule[] $devCtrlSchedules
 * @property DevClasses $parent
 * @property Qrcode $qrcode
 * @property DisableDevSession[] $disableDevSessions
 * @property PlaySessionTime[] $schedules
 * @property MsgTaskDev[] $msgTaskDevs
 * @property MsgTasks[] $msgTasks
 * @property SessionDevice[] $sessionDevices
 * @property Timetable[] $timetables
 * @property UserDevices[] $userDevices
 * @property Users[] $users
 */
class DeviceList extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'DeviceList';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['DevID'], 'required'],
            [['Parent', 'DisableParentSchedule', 'CameraPort', 'CameraType', 'CameraChannel', 'Qrcode'], 'integer'],
            [['DevID'], 'string', 'max' => 50],
            [['Dev_Name'], 'string', 'max' => 32],
            [['Dev_Description'], 'string', 'max' => 96],
            [['Dev_Configuration'], 'string', 'max' => 1024],
            [['CameraIpAddr', 'CameraUser', 'CameraPasswd'], 'string', 'max' => 45],
            [['RtspUrl', 'RtspUrl1'], 'string', 'max' => 128],
            [['DevID'], 'unique'],
            [['Parent'], 'exist', 'skipOnError' => true, 'targetClass' => DevClasses::className(), 'targetAttribute' => ['Parent' => 'ClassID']],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'DevID' => 'Dev ID',
            'Dev_Name' => 'Dev  Name',
            'Parent' => 'Parent',
            'DisableParentSchedule' => 'Disable Parent Schedule',
            'Dev_Description' => 'Dev  Description',
            'Dev_Configuration' => 'Dev  Configuration',
            'CameraIpAddr' => 'Camera Ip Addr',
            'CameraPort' => 'Camera Port',
            'CameraUser' => 'Camera User',
            'CameraPasswd' => 'Camera Passwd',
            'CameraType' => 'Camera Type',
            'CameraChannel' => 'Camera Channel',
            'RtspUrl' => 'Rtsp Url',
            'RtspUrl1' => 'Rtsp Url1',
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBrdModeMembers()
    {
        return $this->hasMany(BrdModeMember::className(), ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getBrdModes()
    {
        return $this->hasMany(BroadcastMode::className(), ['idBroadcastMode' => 'BrdModeID'])->viaTable('BrdModeMember', ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDevCtrlSchedules()
    {
        return $this->hasMany(DevCtrlSchedule::className(), ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getParent()
    {
        return $this->hasOne(DevClasses::className(), ['ClassID' => 'Parent']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getDisableDevSessions()
    {
        return $this->hasMany(DisableDevSession::className(), ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSchedules()
    {
        return $this->hasMany(PlaySessionTime::className(), ['ScheduleID' => 'ScheduleID'])->viaTable('DisableDevSession', ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMsgTaskDevs()
    {
        return $this->hasMany(MsgTaskDev::className(), ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getMsgTasks()
    {
        return $this->hasMany(MsgTasks::className(), ['MsgTaskID' => 'MsgTaskID'])->viaTable('MsgTaskDev', ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSessionDevices()
    {
        return $this->hasMany(SessionDevice::className(), ['DeviceID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getTimetables()
    {
        return $this->hasMany(Timetable::className(), ['DevId' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUserDevices()
    {
        return $this->hasMany(UserDevices::className(), ['DevID' => 'DevID']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getUsers()
    {
        return $this->hasMany(Users::className(), ['User_ID' => 'UserID'])->viaTable('UserDevices', ['DevID' => 'DevID']);
    }
}
