<?php
namespace common\models;

use common\classes\Common;
use common\classes\Error;
use yii\base\Model;
use yii\db\Exception;
use yii;


class UploadFile extends Model
{
    const FILE_EXTENSIONS = 'mp3,ts,mp4,3gp,mov,avi,mpg';
    const FILE_MAX_COUNT = 10;

    /**
     * @var  multiple files instance
     */
    public $multiple;

    /**
     * @return array the validation rules.
     */
    public function rules()
    {
        return [
             [['multiple'], 'file', 'skipOnEmpty' => false, 'extensions' => self::FILE_EXTENSIONS, 'maxFiles' => self::FILE_MAX_COUNT, 'maxSize' => 4294967295, 'checkExtensionByMimeType' => false],
        ];
    }

    //upload multiple files
    public function uploadMultiple($parentid)
    {
         if ($this->validate())
        {
            $files = [];
            foreach ($this->multiple as $file) {
                $filepath = Common::MEDIA_TEMPORARA_DIR . Yii::$app->security->generateRandomString(16);
                $file->saveAs($filepath);
                $streams = Yii::$app->media->info($filepath);
                $media = null;
                do{
                    if(empty($streams)) {
                        $ret = Error::STATUS_MEDIA_GET_INFO_FAILED;
                        break;
                    }
                    $ret = Yii::$app->media->codecCheck($filepath, $streams);
                    if ($ret != Error::STATUS_SUCCESS)
                        break;
                    $media = Yii::$app->media->saveAs($file->name, $filepath, $streams, $parentid);
                    if(empty($media))
                        $ret = Error::STATUS_FAILED;
                }while(false);
                $files[$file->name] = [
                    'error' => $ret,
                    'media' => $media,
                    'message' => Yii::t('app/error', $ret)
                ];
            }
            return $files;
        } else {
            return [];
        }
    }
}
