<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/24 14:54
 */
namespace Workerman\iDste;
use common\classes\Common;
use common\models\CallWaiter;
use common\models\DeviceList;
use common\models\IpCall;
use common\models\UserDevices;
use common\models\Users;
use common\models\VIpCall;
use common\models\VPlayingSessions;
use common\classes\Error;
use common\models\VDeviceMessage;
use Workerman\Lib\Timer;
use Workerman\Worker;
use yii\db\Exception;
use yii\helpers\Json;
use Yii;

require_once __DIR__ . '/../Autoloader.php';
require_once __DIR__ . '/../../common/classes/System.php';
require_once __DIR__ . '/../../common/classes/HttpRequest.php';
require_once __DIR__ . '/../../common/classes/Error.php';

//websocket deamon
class WsDeamon
{
    const KEEPALIVE_TIMER_PERIOD = 1;
    const KEEPALIVE_PERIOD = 30;
    const KEEPALIVE_TIMER_CHANGE_THRESHOLD = 3;
    private $lastId = -1;
    private $lastAliveTime = 0;
    private $lastCheckTime = 0;
    private $clients = [];
    private $loginConns = [];
    private $status = [
        'connCnt' => [
            'device' => 0,
            'session' => 0,
            'ipcall' => 0,
            'player' => 0
        ]
    ];
    private $loginRequestController = ['device', 'session'];

    public function __construct($wsWorker)
    {
        $wsWorker->onWorkerStart = [$this, 'onWorkerStart'];
        $wsWorker->onConnect = [$this, 'onConnect'];
        $wsWorker->onMessage = [$this, 'onMessage'];
        $wsWorker->onClose = [$this, 'onClose'];
    }

    private function response($conn, $status, $data = null, $closeOnError = true) {
        $msg = Error::jsonResponse($status, $data);
        echo "send msg to client:$conn->id, status:$status\n";
        $conn->send($msg);
        if($closeOnError && $status != Error::STATUS_SUCCESS && $status < Error::STATUS_IDSTE_CODE_BEGIN)
            $conn->close();
        return '';
    }
    /*
     * websocket on worker start handle
     */
    public function onWorkerStart($worker)
    {
        echo "websocket worker start\n";
        $this->dbReconnect();

        Timer::add(1, [$this, 'deviceDaemon']);
        Timer::add(1, [$this, 'sessionDaemon']);
        Timer::add(1, [$this, 'ipcallDaemon']);
        Timer::add(1, [$this, 'playerDaemon']);
        Timer::add(self::KEEPALIVE_TIMER_PERIOD, [$this, 'clientKeepAlive']);
        Timer::add(1, [$this, 'clientLoginCheck']);
        Timer::add(300, [$this, 'dbReconnect']);
    }

    /**
     * reconnect every 300 sec
     */
    public function dbReconnect()
    {
        while (true) {
            try {
                Yii::$app->db->close();
                Yii::$app->db->open();
                return;
            } catch (Exception $e) {
                echo "db error, wait. \n";
                sleep(1);
            }
        }
    }

    /*
     * websocket event handle: websocket client connect to server
     */
    public function onConnect($conn)
    {
        echo "onConnect:$conn->id\n";
        $this->loginConns[$conn->id] = [
            'at' => time(),
            'conn' => $conn
        ];
    }

    /**
     * @param $conn
     */
    public function clientLoginCheck() {
        $now = time();
        foreach ($this->loginConns as $id => $item) {
            if($now - $item['at'] > 5 || $now < $item['at']) {
                $this->response($item['conn'], Error::STATUS_ERROR_408);
            }
        }
    }

    public function siteController($conn, $action, $data) {
        switch ($action) {
            case 'login':
                if(empty($data) || empty($data->token) || !is_string($data->token) ||empty($data->type) || !is_string($data->type) || !in_array($data->type, ['login', 'unionid-login', 'oauth']))
                    return $this->response($conn, Error::STATUS_PARAM_ERROR);
                $cred = Yii::$app->getCache()->get(Common::LOGIN_TOKEN_CACHE_SALT . $data->token);
                if (empty($cred) || !isset($cred['user']))
                    return $this->response($conn, Error::STATUS_ERROR_401);
                $this->clients[$conn->id] = [
                    'conn' => $conn,
                    'uid' => $cred['user']->User_ID,
                    'token' => $data->token,
                    'type' => $data->type,
                    'init' => true,
                    'device' => [
                        'listen' => false,
                    ],
                    'session' => [
                        'listen' => false,
                        'init' => true,
                    ],
                    'ipcall' => [
                        'listen' => false,
                        'call' => null
                    ],
                    'player' => [
                        'listen' => false,
                        'init' => true,
                    ]
                ];
                unset($this->loginConns[$conn->id]);
                echo "client:$conn->id send site/login request\n";
                return $this->response($conn, Error::STATUS_SUCCESS, ['time' => Yii::$app->base->date, 'weekday' => Yii::$app->base->weekday]);
                break;
        }
        return $this->response($conn, Error::STATUS_ERROR_404);
    }

    public function deviceController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['device']['listen'] = true;
                $this->clients[$conn->id]['device']['init'] = true;
                $this->clients[$conn->id]['session']['listen'] = false;
                $this->status['connCnt']['device']++;
                echo "client:$conn->id send device/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['device']['listen'] = false;
                echo "client:$conn->id send device/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }

    public function sessionController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['device']['listen'] = false;
                $this->clients[$conn->id]['device']['init'] = true;
                $this->clients[$conn->id]['session']['listen'] = true;
                $this->clients[$conn->id]['session']['init'] = true;
                $this->status['connCnt']['session']++;
                echo "client:$conn->id send sission/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['session']['listen'] = false;
                echo "client:$conn->id send sission/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }


    public function ipcallController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['ipcall']['listen'] = true;
                $this->clients[$conn->id]['ipcall']['call'] = null;
                $this->status['connCnt']['ipcall']++;
                echo "client:$conn->id send sission/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['ipcall']['listen'] = false;
                $this->clients[$conn->id]['ipcall']['call'] = null;
                echo "client:$conn->id send sission/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }


    public function playerController($conn, $action, $data) {
        switch ($action) {
            case 'listen':
                $this->clients[$conn->id]['player']['listen'] = true;
                $this->clients[$conn->id]['player']['init'] = true;
                $this->status['connCnt']['player']++;
                echo "client:$conn->id send player/listen request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
            case 'close':
                $this->clients[$conn->id]['player']['listen'] = false;
                echo "client:$conn->id send player/close request\n";
                return $this->response($conn, Error::STATUS_SUCCESS);
                break;
        }
        $this->response($conn,Error::STATUS_ERROR_404);
    }

    private function run($conn, $route, $data) {
        $route = preg_split('/\//', trim($route), 2);
        if(count($route) != 2)
            return $this->response($conn, Error::STATUS_PARAM_ERROR);

        if(in_array($route[0], $this->loginRequestController) && !isset($this->clients[$conn->id]))
            return $this->response($conn, Error::STATUS_ERROR_401);

        $controller = $route[0] . 'Controller';
        if(in_array($controller, get_class_methods($this)))
            call_user_func_array ([$this, $controller], [$conn, $route[1], $data]);
        else
            $this->response($conn,Error::STATUS_ERROR_404);
    }

    /*
     * send keepalive to all client
     */
    public function clientKeepAlive()
    {
        $timeNow = microtime(true);
        if ($timeNow - $this->lastCheckTime > self::KEEPALIVE_TIMER_CHANGE_THRESHOLD ||
            $this->lastCheckTime > $timeNow ||
            $timeNow - $this->lastAliveTime >= self::KEEPALIVE_PERIOD
        ) {
            $this->lastAliveTime = $timeNow;
            $msg = Error::jsonResponse(Error::STATUS_SUCCESS, ['time' => Yii::$app->base->date, 'weekday' => Yii::$app->base->weekday]);
            foreach ($this->clients as $conn) {
                $conn['conn']->send($msg);
            }
        }
        $this->lastCheckTime = $timeNow;
    }

    /*
     * websocket event handle:receive message from client
     * $data = [
     *      'route' => message route like 'site/login' or 'device/listen' or 'session/listen' and so on.
     *      'data' =>
     * ]
     */
    public function onMessage($conn, $data)
    {
        $msg = json_decode($data);
        if (!empty($msg) && isset($msg->route)) {
            if(empty($msg->data))
                $msg->data  = null;
            $this->run($conn, $msg->route, $msg->data);
        }
        else {
            $this->response($conn, Error::jsonResponse(Error::STATUS_ERROR_404));
        }
    }

    /*
     * websocket event handle: close
     */
    public function onClose($conn)
    {
        echo "close connect:$conn->id\n";
        unset($this->clients[$conn->id]);
        unset($this->loginConns[$conn->id]);
    }

    private $userDevice = [];
    private function getUserDevice($uid)
    {
        $now = time();
        if(isset($this->userDevice[$uid]) && ($now - $this->userDevice[$uid]['at'] < 10) && ($now > $this->userDevice[$uid]['at']))
            return $this->userDevice[$uid]['devices'];

        $user = Users::find()->select(['User_ID', 'Mgr_Privilege', 'privilege'])->where(['User_ID' => $uid])->one();
        //user not exists or has no brd privilege
        if (empty($user) || ($user->User_ID != 0 && (($user->privilege & 0x100) != 0x100)))
            return [];
        if ($user->Mgr_Privilege == 1 || $user->User_ID == 0) {
            $models = DeviceList::find()->select(['DevID'])->all();
        } else {
            $models = UserDevices::find()->select(['DevID'])
                ->where(['UserID' => $uid])
                ->andWhere('Pri & 2 = 2')
                ->all();
        }
        $devices = [];
        foreach ($models as $model) {
            $devices[] = $model->DevID;
        }
        $this->userDevice[$uid] = [
            'at' => $now,
            'devices' => $devices
        ];
        return $devices;
    }

    /*
     * ge message and send to clients
     */
    private $deviceCache = [];
    public function deviceDaemon()
    {
        if($this->status['connCnt']['device'] == 0)
            return;
        $models = VDeviceMessage::find()
            ->where('message > ' . $this->lastId)
            ->orderBy(['message' => SORT_DESC])
            ->groupBy(['id'])
            ->asArray()
            ->all();

        if(empty($models))
            return;

        if ($this->lastId < $models[0]['message'])
            $this->lastId = $models[0]['message'];

        $msgs = [];
        foreach ($models as $model) {
            $model['actualSession'] = is_null($model['actualSession']) ? null : intval($model['actualSession']);
            $model['audioVol'] = is_null($model['audioVol']) ? null : intval($model['audioVol']);
            $model['fault'] = is_null($model['fault']) ? null : intval($model['fault']);
            $model['micVol'] = is_null($model['micVol']) ? null : intval($model['micVol']);
            $model['parent'] = is_null($model['parent']) ? null : intval($model['parent']);
            $model['shouldSession'] = is_null($model['shouldSession']) ? null : intval($model['shouldSession']);
            unset($model['message']);
            $msgs[$model['id']] = $model;
        }
        $connCnt = 0;
        foreach ($this->clients as $clientid => $client) {
            if (!$client['device']['listen'])
                continue;
            $connCnt++;
            $uid = $client['uid'];
            $devices = $this->getUserDevice($uid);
            if (empty($devices))
                continue;
            $ret = [];
            foreach ($devices as $id) {
                if(!isset($msgs[$id])) {
                    if(isset($this->deviceCache[$id]) && $client['device']['init'])
                        $ret[] = array_merge($this->deviceCache[$id], ['status' => 'change']);
                }
                else {
                    if (!isset($this->deviceCache[$id]) || $client['device']['init']) {
                        $ret[] = array_merge($msgs[$id], ['status' => 'change']);
                    } else {
                        $diff = array_diff_assoc($msgs[$id], $this->deviceCache[$id]);
                        if (empty($diff))
                            continue;
                        $ret[] = array_merge($diff, ['status' => 'change', 'id' => $id]);
                    }
                }
            }
            $this->clients[$clientid]['device']['init'] = false;
            if (empty($ret))
                continue;
            $this->response($client['conn'], Error::STATUS_DEVICE_CHANGED, ['diff' => $ret]);
        }
        foreach ($msgs as $id => $msg)
            $this->deviceCache[$id] = $msg;
        $this->status['connCnt']['device'] = $connCnt;
        if($connCnt == 0)
            $this->deviceCache = [];
    }

    /**
     * get session status and send to client
     */
    private $sessionCache = [];
    public function sessionDaemon(){
        if($this->status['connCnt']['session'] == 0)
            return;
        $models = VPlayingSessions::find()
            ->select(['SessionID', 'FileName', 'CurrTotalProgress', 'Pause', 'PlayTime', 'WeekDay', 'Template', 'BrdMode', 'SessionType', 'StreamType'])
            ->indexBy('SessionID')
            ->orderBy(['PlayTime' => SORT_ASC])
            ->asArray()
            ->all();

        $diffs = [];
        foreach ($models as $id => $model) {
            $model['CurrTotalProgress'] = is_null($model['CurrTotalProgress']) ? 0 : intval($model['CurrTotalProgress']);
            $model['Pause'] = is_null($model['Pause']) ? null : intval($model['Pause']);
            $model['SessionID'] = intval($model['SessionID']);
            if(isset($this->sessionCache[$id])){
                if($this->sessionCache[$id]['Pause'] == $model['Pause'])
                     $this->sessionCache[$id]['CurrTotalProgress'] = $model['CurrTotalProgress'];
                $diff = array_diff_assoc($model, $this->sessionCache[$id]);
                if (empty($diff))
                    continue;
                $diff['status'] = 'change';
                $diff['PlayTime'] = $model['PlayTime'];
                $diff['CurrTotalProgress'] = $model['CurrTotalProgress'];
                $diff['SessionID'] = $id;
                $diffs[] = $diff;
            }else {
                $model['status'] = 'change';
                $diffs[] = $model;
            }
            $this->sessionCache[$id] = $model;
        }

        foreach ($this->sessionCache as $id => $model) {
            if(!isset($models[$id])) {
                $diffs[] = ['status' => 'stop', 'SessionID' => intval($id), 'PlayTime' => $model['PlayTime']];
                unset($this->sessionCache[$id]);
            }
        }

        $connCnt = 0;
        foreach ($this->clients as $id => $client) {
            if(!$client['session']['listen'])
                continue;
            $connCnt++;
            if(count($this->sessionCache) > 0 && ($client['session']['init'] || (time() % 63 == 0))){
                $this->response($client['conn'], Error::STATUS_SESSION_STATUS_CHANGED, ['flush' => array_values($this->sessionCache)]);
                $this->clients[$id]['session']['init'] = false;
            }
            else if (!empty($diffs))
                $this->response($client['conn'], Error::STATUS_SESSION_STATUS_CHANGED, ['diff' => $diffs]);
        }
        $this->status['connCnt']['session'] = $connCnt;
        if($connCnt == 0)
            $this->sessionCache = [];
    }

    /**
     * fetch player status and send to client
     */
    private $playerCache = [];
    public function playerDaemon(){
        if($this->status['connCnt']['player'] == 0)
            return;
        $models = Yii::$app->db->createCommand('select a.PlayerID, a.SessionID as RealSessionID, a.SessionName, a.Priority, a.AudioGain, a.Owner, a.OwnerName, a.StreamType, a.SessionType,
                                                    c.SessionID, c.ScheduleID, a.PlayerAddress as IpAddress, a.PlayType, a.PlayerType from v_session_info a
                                                    left join PlayerStatus c on c.SessionID  = a.SessionID
                                                    where a.PlayerID is not null;')
            ->queryAll();

        $diffs = [];
        $list = [];
        foreach ($models as $model) {
            $id = $model['PlayerID'];
            $list[$id] = true;
            $model['SessionID'] = intval($model['SessionID']);
            $model['RealSessionID'] = intval($model['RealSessionID']);
            $model['ScheduleID'] = intval($model['ScheduleID']);
            $model['PlayerType'] = intval($model['PlayerType']);
            if(isset($this->playerCache[$id])){
                $diff = array_diff_assoc($model, $this->playerCache[$id]);
                if (empty($diff))
                    continue;
                $diff['status'] = 'change';
                $diff['PlayerID'] = $id;
                $diffs[] = $diff;
            }else {
                $model['status'] = 'change';
                $diffs[] = $model;
            }
            $this->playerCache[$id] = $model;
        }

        foreach ($this->playerCache as $id => $model) {
            if(!isset($list[$id])) {
                $diffs[] = ['status' => 'delete', 'PlayerID' => '' . $id];
                unset($this->playerCache[$id]);
            }
        }

        $connCnt = 0;
        foreach ($this->clients as $id => $client) {
            if(!$client['player']['listen'])
                continue;
            $connCnt++;
            if(count($this->playerCache) > 0 && ($client['player']['init'] || (time() % 61 == 0))){
                $this->response($client['conn'], Error::STATUS_PLAYER_STATUS_CHANGED, ['flush' => array_values($this->playerCache)]);
                $this->clients[$id]['player']['init'] = false;
            }
            else if (!empty($diffs))
                $this->response($client['conn'], Error::STATUS_PLAYER_STATUS_CHANGED, ['diff' => array_values($diffs)]);
        }
        $this->status['connCnt']['player'] = $connCnt;
        if($connCnt == 0)
            $this->playerCache = [];
    }
    /**
     * get ipcall recoder
     */
    public function ipcallDaemon(){
        if($this->status['connCnt']['ipcall'] == 0)
            return;

        $connCnt = 0;
        foreach ($this->clients as $id => $client){
            if(!$client['ipcall']['listen'])
                continue;
            $connCnt++;
            Yii::$app->db
                ->createCommand("insert into CallWaiter (user, session) values(" . $client['uid'] . ", '" . $client['token'] . "') on duplicate key update updateAt= now()")
                ->execute();
            $waiter = CallWaiter::findOne(['user' => $client['uid'], 'session' => $client['token']]);
            if(empty($waiter)){
                $this->clients[$id]['ipcall']['call'] = null;
                continue;
            }

            $query = VIpCall::find()
                ->select(['id', 'device', 'deviceName',  'waiter', 'waiterName', 'ringing', 'status', 'direction', 'wsPort']);
            if(!empty($client['ipcall']['call']))
                $query->where(['id' => $client['ipcall']['call']['id']]);
            else
                $query->where("`waiter` is null and `status`='ringing'");
            $call = $query->orderBy(['id' => SORT_ASC])
                ->asArray()
                ->one();
            $ret = [];
            if(!empty($call)) {
                $call['id'] = intval($call['id']);
                $call['wsPort'] = intval($call['wsPort']);
                if((!is_null($call['waiter']) && $call['waiter'] != $waiter->id) || $call['status'] == 'idle') {
                    $ret[] = [
                        'id' => $call['id'],
                        'status' => 'idle'
                    ];
                    $call = null;
                }
                else {
                    unset($call['waiter']);
                    if(!empty($client['ipcall']['call']) && (time() % 10 != 0)) {
                        $tmp = array_diff_assoc($call, $client['ipcall']['call']);
                        if (empty($tmp))
                            continue;
                        $tmp['id'] = $call['id'];
                        $ret[] = $tmp;
                    }
                    else
                        $ret[] = $call;
                }
            }
            $this->clients[$id]['ipcall']['call'] = $call;
            if(!empty($ret))
                $this->response($client['conn'], Error::STATUS_IPCALL_STATE_CHANGED, $ret);
        }
        $this->status['connCnt']['ipcall'] = $connCnt;
    }
}
