<?php
/**
 * User: xuhj(xuhj@idste.cn)
 * Date: 2018/1/24 14:54
 */
namespace Workerman\iDste;
use common\models\SystemLog;
use Workerman\Worker;
use Yii;

use common\classes\Error;
use common\classes\HttpRequest;
use common\classes\System;
use common\models\BaseInfo;
use Workerman\Lib\Timer;
use yii\db\Exception;
use yii\db\Query;

require_once __DIR__ . '/../Autoloader.php';
require_once  __DIR__ . '/../../common/classes/System.php';
require_once  __DIR__ . '/../../common/classes/HttpRequest.php';
require_once  __DIR__ . '/../../common/classes/Error.php';
require_once  __DIR__ . '/SystemDeamon.php';


class iDsteDeamon
{
    private $userMagic;
    private $baseInfo;
    private $deamonWorker;
    private $updateUserLastTime;
    private $proxyConnected = false;
    private $systemDeamon;
    public function __construct($deamon){
        $this->userMagic = -1;
        $this->updateUserLastTime = 0;
        $this->baseInfo = null;
        $this->deamonWorker = $deamon;
        $deamon->onWorkerStart = [$this, 'workerStart'];
        shell_exec('chown www-data:www-data /dev/shm/yii2cache/ -R');
    }

    public function workerStart($worker){
        echo "idste worker start\n";
        $this->dbReconnect();
        if($worker->id == 0)
            $this->idsteDeamonInit();
        else if($worker->id == 1)
            $this->systemDeamon = new SystemDeamon();
    }

    private function idsteDeamonInit(){
        $this->baseInfo['mac'] = System::getMac();
        while(true) {
            $this->baseInfo['uuid'] = Yii::$app->base->uuid;
            //uuid
            if (empty($this->baseInfo['uuid'])) {
                echo "unregisted server\n";
                sleep(30);
                continue;
            }
            $version = Yii::$app->base->version;
            $this->baseInfo['version'] = str_replace('.', '', $version);
            $this->baseInfo['server'] = 'https://' . Yii::$app->base->cloudServer;
            //secret_token
            $this->baseInfo['secret_token'] = Yii::$app->base->secretToken;
            //access_token
            $this->baseInfo['access_token'] = Yii::$app->base->accessToken;
            if (empty($this->baseInfo['access_token']))
                $this->baseInfo['access_token'] = 'empty';
            break;
        }
        $this->proxyAlive();
        Timer::add(30, [$this, 'proxyAlive'], []);
        Timer::add(300, [$this, 'dbReconnect']);
        Timer::add(2, [$this, 'userSync']);
    }

    /**
     * reconnect every 300 sec
     */
    public function dbReconnect(){
        while (true) {
            try {
                Yii::$app->db->close();
                Yii::$app->db->open();
                return;
            } catch (Exception $e) {
                echo "db error, wait. \n";
                sleep(1);
            }
        }
    }

    /*
     * send confirm request
     */
    private function proxyConfirm($data){
        if(!isset($data->secret_token) || !isset($data->access_token) || !isset($data->origin))
            return;

        $arg = null;
        $arg['data']['secret_token'] = $data->secret_token;
        $arg['data']['access_token'] = $data->access_token;
        $req = new HttpRequest($this->baseInfo['server'] . '/api/school/token-confirm',
            ['uuid' => $this->baseInfo['uuid']]);
        $ret = $req->post(json_encode($arg), false);
        if(is_null($ret) || !isset($ret->status) || $ret->status !== 0) {
            echo "proxy confirm failed:" . json_encode($ret) . "\n";
            return;
        }

        $this->baseInfo = array_merge($this->baseInfo, $arg['data']);
        $db = \Yii::$app->db;
        $db->createCommand("insert into BaseInfo set `InfomationID` = 338, `Description`='" .$data->secret_token. "', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
            ->query();
        $db->createCommand("insert into BaseInfo set `InfomationID` = 339, `Description`='" .$data->access_token. "', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
            ->query();
        $db->createCommand("insert into BaseInfo set `InfomationID` = 340, `Description`='" .$data->origin. "', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
            ->query();
    }

    private function reloadToken(){
        $this->baseInfo['access_token'] = Yii::$app->base->accessToken;
        $this->baseInfo['secret_token'] = Yii::$app->base->secretToken;
    }
    /*
     * sync user data
     */
    public function userSync(){
        if(!isset($this->baseInfo['access_token']) || !$this->proxyConnected)
            return;
        $db = \Yii::$app->db;
        $magic = 0;
        $info = BaseInfo::findOne(['InfomationID' => 341]);
        if(empty($info)) {
            $db->createCommand("insert into `BaseInfo` (`InfomationID`, `InfoValue`) values (341, 0) on duplicate key update `InfoValue` = `InfoValue` + 1;")
                ->query();
        }
        else if($this->userMagic == $info->InfoValue && (time() - $this->updateUserLastTime) < 1800)
            return;
        else //userMagic unmatch or need force update
            $magic = $info->InfoValue;
        echo "user sync.\n";
        $teachers = Yii::$app->db->createCommand('select group_concat(`User_ID`) as teachers from Users where UnionID is not null;')
            ->queryOne();
        if(empty($teachers) || is_null($teachers['teachers']))
            $teacherStr = null;
        else
            $teacherStr = $teachers['teachers'];

        $users = (new Query())->select(['pid' => 'User_ID', 'username' => 'User_Name', 'fullname' => 'User_Desc', 'phone' => 'Phone', 'unionid' => 'UnionId', 'privilege'])
            ->from('Users')
            ->where(['not', ['User_Name' => null]])
            ->andWhere(['not', ['Pass_Word' => null]])
            ->all();
        $req = new HttpRequest($this->baseInfo['server'] . '/api/school/user-sync', [
            'access_token' => $this->baseInfo['access_token'],
            'uuid' => $this->baseInfo['uuid']]);
        $out = $req->post(['data' => $users, 'teacher' => $teacherStr, 'magic' => $magic]);
        if(isset($out->status) && $out->status == 0){
            $this->updateUserLastTime = time();
            $this->userMagic = $magic;
            if(isset($out->data) && is_array($out->data->unsetUnionid)) {
                $users = implode(',', $out->data->unsetUnionid);
                $db->createCommand("update Users set UnionID = null where User_Name is not null and Pass_Word is not null and User_ID in ($users);")
                    ->query();
            }
        }
        else if($out->status && $out->status = 401)
            $this->reloadToken();
        else
            echo "user sync error:" . print_r($out, true) . "\n";
    }

    /*
     * kill ssh proxy progress
     */
    private function breakProxy(){
        $cmd = "ps aux | grep 'ssh -i " . __DIR__ . "/proxy_user.key' | awk '{print $2}' | xargs kill -9";
        system($cmd . ' > /dev/null');
    }
    /*
     * rebuild ssh proxy
     */
    private function proxyReconnect($data){
        //kill exist connect
        $this->breakProxy();
        //reconnect
        $cmd = 'ssh -i ' . __DIR__ . '/proxy_user.key -o ServerAliveInterval=60 -o ServerAliveCountMax=3 ';
        $cmd .= '-o StrictHostKeyChecking=no -NfR ';
        $cmd .= $data->port .':127.0.0.1:18080 proxy_user@'. $data->host;
        system($cmd . ' > /dev/null &');
        echo "proxy reconnect\n";
        \Workerman\Lib\Timer::add(10, [$this, 'proxyAlive'], [], false);
    }

    /*
     * proxy keepalive every 30sec
     */
    public function proxyAlive(){
        try{
            $db = \Yii::$app->db;
            $req = new HttpRequest($this->baseInfo['server'] . '/api/school/sync',
                ['uuid' => $this->baseInfo['uuid']]);
            $this->baseInfo['net'] = array_values(Yii::$app->net->ipView()['state']);
            $arg['data'] = $this->baseInfo;
            unset($arg['data']['secret_token']);
            $out = $req->post(json_encode($arg), false);
            $proxyConnected = false;
            if(is_null($out) || !isset($out->status)) {
                echo "proxy alive error\npost:" . print_r($arg, true) . "\n";
                echo "out:" . print_r($out, true) . "\n";
                $db->createCommand("insert into BaseInfo set `InfomationID` = 337, `Description`='error', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
                    ->query();
            }
            else if($out->status == 0) {
                $proxyConnected = true;
                if(isset($out->data))
                    $this->proxyConfirm($out->data, $arg['data']['uuid']);
                $db->createCommand("insert into BaseInfo set `InfomationID` = 337, `Description`='success', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
                    ->query();
                $this->userSync();
            }
            else if($out->status == Error::STATUS_PROXY_BROKEN) {
                echo "proxy broken\npost:" . print_r($arg, true) . "\n";
                echo "out:" . print_r($out, true) . "\n";
                $db->createCommand("insert into BaseInfo set `InfomationID` = 337, `Description`='reconnect', `InfoValue` = " . time() . "  on duplicate key update `Description`=VALUES(`Description`), `InfoValue` = VALUES(`InfoValue`);")
                    ->query();
                $this->proxyReconnect($out->data);
            }
            else if($out->status == Error::STATUS_PROXY_OFFLINE){
                echo "proxy offline\npost:" . print_r($arg, true) . "\n";
                echo "out:" . print_r($out, true) . "\n";
            }
            if($this->proxyConnected != $proxyConnected) {
                $this->proxyConnected = $proxyConnected;
            }
        }catch (Exception $e){
            $this->dbReconnect();
            Yii::error("proxy alive exception:" . print_r($e, true));
        }
    }
}
